from pathlib import Path
from dotenv import load_dotenv
import yaml

# ################################################################################
# config_load
# ################################################################################

def find_yml(name):
    p = Path(f'./{name}')
    if p.exists(): return p
    p = Path(f'/run/secrets/{name}')
    if p.exists(): return p
    return None

def config_load(name=None):
    """ Load config yml file. File name default is f'config-{name}.yml' or config.yml by default.
        Checks for file first in the current dir then in /run/secrets """
    load_dotenv()
    name = 'config.yml' if name is None else f'config-{name}.yml'
    fname = find_yml(name)
    if fname is None:
        raise RuntimeError('Unable to find config file')
    with open(str(fname),'r') as stream:
        try:
            cfg = yaml.safe_load(stream)
        except yaml.YAMLError as err:
            raise RuntimeError(f'Unable to load config file: {str(err)}')
    return cfg



#
# DEPRECATED -> REPLACED WITH MQ SERVER
#

import asyncio
import sys
from dl2050utils.core import oget,listify
from dl2050utils.env import config_load
from dl2050utils.log import AppLog
from dl2050utils.db import DB
from dl2050utils.mq import MQ
from dl2050utils.restutils import get_meta

class WorkerServer():
    def __init__(self, product_name, path, qs, cb):
        cfg = config_load(product_name)
        LOG = AppLog(cfg, service='worker')
        db = DB(cfg=cfg, log=LOG)
        mq = MQ(LOG, db, qs, cfg)
        self.cfg,self.LOG,self.db,self.mq = cfg,LOG,db,mq
        self.path,self.qs,self.cb = path,listify(qs),cb
        self.meta = None

    def run(self):
        self.loop = asyncio.get_event_loop()
        if self.db.sync_startup(loop=self.loop):
            self.LOG(4, 0, label='WORKER', label2='STARTUP', msg='DB error')
            exit(1)
        if self.mq.sync_startup(loop=self.loop):
            self.LOG(4, 0, label='WORKER', label2='STARTUP', msg='MQ error')
            exit(1)
        model = oget(self.cfg, ['app','model'])
        if model is not None:
            self.meta = self.loop.run_until_complete(get_meta(self.path, self.db, model))
        self.LOG(2, 0, label='WORKER', label2='RUN', msg='OK')
        err = self.loop.run_until_complete(self.mq.consumer(self, self.qs[0], self.cb))
        if err:
            self.LOG(4, 0, label='WORKER', label2='EXCEPTION', msg='Exit')
            sys.exit(1)

    def job_start(self,*args,**kwargs): return self.loop.run_until_complete(self.mq.job_start(*args,**kwargs))
    def update_eta(self,*args,**kwargs): return self.loop.run_until_complete(self.mq.job_update_eta(*args,**kwargs))
    def job_done(self,*args,**kwargs): return self.loop.run_until_complete(self.mq.job_done(*args,**kwargs))
    def job_notify(self,*args,**kwargs): return self.loop.run_until_complete(self.mq.job_notify(*args,**kwargs))
    def job_deliver(self,*args,**kwargs): return self.loop.run_until_complete(self.mq.job_deliver(*args,**kwargs))
    def job_result(self,*args,**kwargs): return self.loop.run_until_complete(self.mq.job_result(*args,**kwargs))
    def job_error(self,*args,**kwargs): return self.loop.run_until_complete(self.mq.job_error(*args,**kwargs))
    def get_pending_jobs(self,*args,**kwargs): return self.loop.run_until_complete(self.mq.get_pending_jobs(*args,**kwargs))
    def get_job(self,*args,**kwargs): return self.loop.run_until_complete(self.mq.get_job(*args,**kwargs))



# -*- coding: utf-8 -*-
#
# setup.py
# Create source distribution with: python setup.py sdist
#
from distutils.core import setup
from dl2050utils.__config__ import name, package, description, author, author_email, keywords, get_camel
from dl2050utils.__version__ import version

version_camel = get_camel(version)

url = f'https://github.com/jn2050/{name}'
download_url = f'https://github.com/jn2050/{name}/archive/v_{version_camel}.tar.gz'
classifiers=[
      'Development Status :: 4 - Beta', # "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
      'Intended Audience :: Developers',
      'Topic :: Software Development :: Build Tools',
      'License :: OSI Approved :: MIT License',
      'Programming Language :: Python :: 3.7',
]

setup(name=package,
      packages=[package],
      version=version,
      license='MIT',
      description=description,
      author=author,
      author_email=author_email,
      keywords=keywords,
      url=url,
      download_url=download_url,
      classifiers=classifiers
)
